/*
 * deskicon.cc - widget code for icons not bound to a window
 *
 * This file is part of the source distribution of 
 * Jafm - an icon based X11 desktop program 
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 */

#include <qpainter.h>
#include <qpopmenu.h>

#include "deskicon.hh"
#include "icon.hh"
#include "sfm.hh"

#include <X11/Xlib.h>
#include <X11/extensions/shape.h>


DeskIcon::DeskIcon (Icon *ic, bool temp = true )
  : QWidget (0, 0, WStyle_Customize | WStyle_NoBorder) 
{
  
  icon = new Icon (ic) ; // copy what we were given
  transient = temp ;
  if (!temp) { 
    connect ((Sfm *)qApp, SIGNAL (dragDropHighlight (WId, const QPoint &, const QList<DeskIcon> &)),
			this, SLOT (dragDropHighlight (WId, const QPoint &, const QList<DeskIcon> &))) ; 
    connect ((Sfm *)qApp, SIGNAL (drag_drop_happened (WId, const QPoint &, const QList<DeskIcon> &)),
			this, SLOT (drag_drop_happened (WId, const QPoint &, const QList<DeskIcon> &))) ; 
    connect (this, SIGNAL (newDirWindow (const char *)),
	     (Sfm *)qApp, SLOT (open (const char *))) ;

    // make a new popup menu
    menu = new QPopupMenu ;
    menu->insertItem ("vanish") ;
    menu->insertItem ("larger", (QObject *)this, SLOT (larger())) ;
    menu->insertItem ("smaller", (QObject *)this, SLOT (smaller ())) ;


  }
  icon->move (0, 0) ;
  setBackgroundColor (white) ;
  setFocusPolicy (StrongFocus) ;
  set_shape () ;
  setFocus () ;
}
DeskIcon::DeskIcon (DeskIcon *di, bool temp=true)
  : QWidget (0, 0, WStyle_Customize | WStyle_NoBorder) 
{
  
  icon = di->icon ; // copy what we were given
  transient = temp ;
  if (!temp) { 
    connect ((Sfm *)qApp, SIGNAL (dragDropHighlight (WId, const QPoint &, const QList<DeskIcon> &)),
			this, SLOT (dragDropHighlight (WId, const QPoint &, const QList<DeskIcon> &))) ; 
    connect ((Sfm *)qApp, SIGNAL (drag_drop_happened (WId, const QPoint &, const QList<DeskIcon> &)),
			this, SLOT (drag_drop_happened (WId, const QPoint &, const QList<DeskIcon> &))) ; 
    connect (this, SIGNAL (newDirWindow (const char *)),
	     (Sfm *)qApp, SLOT (open (const char *))) ;
    menu = new QPopupMenu ;
    menu->insertItem ("vanish") ;
    menu->insertItem ("larger", (QObject *)this, SLOT (larger())) ;
    menu->insertItem ("smaller", (QObject *)this, SLOT (smaller ())) ;

  }
  icon->move (0, 0) ;
  setFocusPolicy (StrongFocus) ;
  set_shape () ;
  setFocus () ;
  move (di->pos()) ;
}

void DeskIcon::set_shape ()
{
  
  // this is the icon shape
  XShapeCombineMask (x11Display(), handle(),
		     ShapeBounding, 0, 0, icon->get_mask()->handle(),
		     ShapeSet) ; 
  //  setGeometry (icon->rect()) ; // Qt doesn't know we used XShape!
  // make room for label
  QRect *r = &icon->get_label_rect () ;
  QBitmap bm (r->width(), r->height()) ;
  QPainter p (&bm) ;
  p.fillRect (0,0,r->width(),r->height(), color1) ;
  int xoff = (icon->width() - r->width())/2 ;
  // r->moveBy (-xoff, 0) ;
  XShapeCombineMask (x11Display(), handle(),
		     ShapeBounding, xoff, icon->height(), bm.handle(),
		     ShapeUnion) ;
  // set qt's idea of geometry
  int bShaped, xbs, ybs, cShaped, xcs, ycs ;
  uint wbs, hbs, wcs, hcs ;   
  XShapeQueryExtents (x11Display (), handle (),
		      &bShaped, &xbs, &ybs,
		      &wbs, &hbs, &cShaped, 
		      &xcs, &ycs, &wcs, &hcs) ;
  //  resize (wbs, hbs) ;		      
}

void DeskIcon::paintEvent (QPaintEvent *pe)
{
  QPainter p(this) ;
  icon->draw (p) ;
}

void DeskIcon::focusInEvent(QFocusEvent *)
{
  icon->take_focus () ;
  repaint () ;
}

void DeskIcon::focusOutEvent (QFocusEvent *)
{
  icon->drop_focus () ;
  repaint () ;
}

void DeskIcon::mousePressEvent (QMouseEvent *me)
{
  if (me->button() == RightButton) {
    menu->popup (mapToGlobal(me->pos())) ;
    return ;
  }



  pressPoint = me->pos() ;
  raise () ;
}

void DeskIcon::mouseMoveEvent (QMouseEvent *me)
{
  // assume we are dragging
  
  move (mapToGlobal(me->pos ()-pressPoint)) ;
}

void DeskIcon::mouseDoubleClickEvent (QMouseEvent *me)
{
  if (icon->is_dir ()) {
    emit newDirWindow (icon->get_full_name()) ;
  } else {
    icon->open_action () ;
  }
}

void DeskIcon::dragDropHighlight (WId w, const QPoint &p, const QList<DeskIcon> &)
{
  if (transient) return ;
  if (w == winId()) {
    if (geometry().contains (p)) {
      if (icon->is_drag_drop()) return ;
      icon->set_drag_drop (true) ;
      repaint () ;
    }
    return ;
  } 
  if (icon->is_drag_drop()) {
    icon->set_drag_drop (false) ;
    repaint () ;
  }
}

void DeskIcon::drag_drop_happened (WId w, const QPoint &, const QList<DeskIcon> &diList) 
{
  // if WId is us, act accordingly
  if (winId() == w) {
    icon->drag_drop_action (diList) ;
    icon->set_drag_drop (false) ;
    repaint () ;
  }
}

void DeskIcon::larger ()
{
  icon->rescale (icon->get_scale()/8+1) ;
  set_shape () ;
  repaint () ;
}


void DeskIcon::smaller ()
{
  icon->rescale (icon->get_scale()/8-1) ;
  set_shape () ;
  repaint () ;
}
